@@ -20,6 +20,7 @@
 
 #include "gso_interface.h"
 #include "nr/matrix.h"
+#include "util.h"
 
 FPLLL_BEGIN_NAMESPACE
 
@@ -137,10 +138,12 @@ public:
    */
   Matrix<ZT> g;
 
+  virtual inline ZT &sqnorm_coordinates(ZT &sqnorm, vector<ZT> coordinates);
+
   virtual inline long get_max_exp_of_b();
   virtual inline bool b_row_is_zero(int i);
-  virtual inline int get_cols_of_b();
-  virtual inline int get_rows_of_b();
+  virtual inline int get_cols_of_b() const;
+  virtual inline int get_rows_of_b() const;
   virtual inline void negate_row_of_b(int i);
 
   virtual inline void create_rows(int n_new_rows);
@@ -178,9 +181,58 @@ public:
   //  virtual inline void printparam(ostream &os);
   virtual inline FT &get_gram(FT &f, int i, int j);
 
+  virtual inline ZT &get_int_gram(ZT &z, int i, int j);
+
   // b[i] <-> b[j] (i < j)
   virtual void row_swap(int i, int j);
 
+  /**
+     Given a vector `v` wrt the Gram-Schmidt basis `B^*` return a vector `w` wrt the
+     canonical basis `ZZ^n`, i.e. solve `w = v⋅B^*`.
+
+     @param v      a tuple-like object of dimension ``M.d``
+     @param start  only consider subbasis starting at ``start``
+
+   */
+
+  void to_canonical(vector<FT> &w, const vector<FT> &v, long start = 0);
+
+  /**
+     Given a vector `w` wrt the canonical basis `ZZ^n` return a vector `v` wrt the Gram-Schmidt
+     basis `B^*`.
+
+     @param v          a tuple-like object of dimension ``M.B.ncols``
+     @param start      only consider subbasis starting at ``start``
+     @param dimension: only consider ``dimension`` vectors or all if ``-1``
+
+   */
+
+  void from_canonical(vector<FT> &v, const vector<FT> &w, long start = 0, long dimension = -1);
+
+  /**
+   * Update`v` to `w` s.t. `‖w⋅B - v‖` is small using Babai's nearest plane algorithm.
+   *
+   * @param v         a vector
+   * @param start     only consider subbasis starting at start
+   * @param dimension only consider dimension vectors or all if -1
+   * @param gso       if true then input `v` is considered to be expressed to `B*`
+   */
+
+  void virtual babai(vector<ZT> &v, int start = 0, int dimension = -1, bool gso = false);
+
+  /**
+   * Return vector `w` s.t. `‖w⋅B - v‖` is small using Babai's nearest plane algorithm.
+   *
+   * @param w         a vector
+   * @param v         a vector
+   * @param start     only consider subbasis starting at start
+   * @param dimension only consider dimension vectors or all if -1
+   * @param gso       if true then input `v` is considered to be expressed wrt `B*`
+   */
+
+  void virtual babai(vector<ZT> &w, const vector<FT> &v, int start = 0, int dimension = -1,
+                     bool gso = false);
+
 private:
   /* Allocates matrices and arrays whose size depends on d (all but tmp_col_expo).
    When enable_int_gram=false, initializes bf. */
@@ -201,6 +253,21 @@ private:
   virtual void row_addmul_2exp(int i, int j, const ZT &x, long expo);
 };
 
+template <class ZT, class FT>
+inline ZT &MatGSO<ZT, FT>::sqnorm_coordinates(ZT &sqnorm, vector<ZT> coordinates)
+{
+  vector<ZT> tmpvec;
+  ZT tmp;
+  sqnorm = 0;
+  vector_matrix_product(tmpvec, coordinates, b);
+  for (size_t j = 0; j < tmpvec.size(); j++)
+  {
+    tmp.mul(tmpvec[j], tmpvec[j]);
+    sqnorm.add(sqnorm, tmp);
+  }
+  return sqnorm;
+}
+
 template <class ZT, class FT> inline long MatGSO<ZT, FT>::get_max_exp_of_b()
 {
   return b.get_max_exp();
@@ -210,9 +277,15 @@ template <class ZT, class FT> inline bool MatGSO<ZT, FT>::b_row_is_zero(int i)
 {
   return b[i].is_zero();
 }
-template <class ZT, class FT> inline int MatGSO<ZT, FT>::get_cols_of_b() { return b.get_cols(); }
+template <class ZT, class FT> inline int MatGSO<ZT, FT>::get_cols_of_b() const
+{
+  return b.get_cols();
+}
 
-template <class ZT, class FT> inline int MatGSO<ZT, FT>::get_rows_of_b() { return b.get_rows(); }
+template <class ZT, class FT> inline int MatGSO<ZT, FT>::get_rows_of_b() const
+{
+  return b.get_rows();
+}
 
 template <class ZT, class FT> inline void MatGSO<ZT, FT>::negate_row_of_b(int i)
 {
@@ -249,13 +322,29 @@ template <class ZT, class FT> inline FT &MatGSO<ZT, FT>::get_gram(FT &f, int i,
   {
     if (gf(i, j).is_nan())
     {
-      dot_product(gf(i, j), bf[i], bf[j], n_known_cols);
+      bf[i].dot_product(gf(i, j), bf[j], n_known_cols);
     }
     f = gf(i, j);
   }
   return f;
 }
 
+template <class ZT, class FT> inline ZT &MatGSO<ZT, FT>::get_int_gram(ZT &z, int i, int j)
+{
+  FPLLL_DEBUG_CHECK(i >= 0 && i < n_known_rows && j >= 0 && j <= i && j < n_source_rows &&
+                    !in_row_op_range(i));
+  if (enable_int_gram)
+  {
+    z = g(i, j);
+  }
+  else
+  {
+
+    b[i].dot_product(z, b[j], n_known_cols);
+  }
+  return z;
+}
+
 template <class ZT, class FT> inline void MatGSO<ZT, FT>::create_rows(int n_new_rows)
 {
   FPLLL_DEBUG_CHECK(!cols_locked);
@@ -274,7 +363,7 @@ template <class ZT, class FT> inline void MatGSO<ZT, FT>::create_rows(int n_new_
     u.set_rows(d);
     for (int i = old_d; i < d; i++)
       for (int j = 0; j < u.get_cols(); j++)
-        u[i][j]  = 0;
+        u[i][j] = 0;
   }
   size_increased();
   if (n_known_rows == old_d)
