@@ -98,7 +98,8 @@
 #define DPE_EXPMIN INT_MIN /* smallest possible exponent */
 #endif
 
-typedef union {
+typedef union
+{
   double d;
   int i[2];
 } dpe_double_words;
