@@ -1,5 +1,6 @@
 /* Copyright (C) 2008-2011 Xavier Pujol.
     (C) 2015 Michael Walter.
+    Copyright (C) 2019 Koen de Boer & Wessel van Woerden
 
    This file is part of fplll. fplll is free software: you
    can redistribute it and/or modify it under the terms of the GNU Lesser
@@ -57,45 +58,26 @@ static void get_basis_min(Z_NR<mpz_t> &basis_min, const ZZ_mat<mpz_t> &b, int fi
   }
 }
 
-static bool enumerate_svp(int d, MatGSO<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso, FP_NR<mpfr_t> &max_dist,
-                          ErrorBoundedEvaluator &evaluator, const vector<enumf> &pruning, int flags)
+static void get_basis_min(Z_NR<mpz_t> &basis_min, MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso,
+                          int first, int last)
+{
+  Z_NR<mpz_t> sq_norm;
+  gso.get_int_gram(basis_min, first, first);
+  for (int i = first + 1; i < last; i++)
+  {
+    gso.get_int_gram(sq_norm, i, i);
+    if (sq_norm < basis_min)
+      basis_min = sq_norm;
+  }
+}
+
+static bool enumerate_svp(int d, MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso,
+                          FP_NR<mpfr_t> &max_dist, ErrorBoundedEvaluator &evaluator,
+                          const vector<enumf> &pruning, int flags)
 {
   Enumeration<Z_NR<mpz_t>, FP_NR<mpfr_t>> enumobj(gso, evaluator);
   bool dual = (flags & SVP_DUAL);
-  if (d == 1 || !pruning.empty() || dual)
-  {
-    enumobj.enumerate(0, d, max_dist, 0, vector<FP_NR<mpfr_t>>(), vector<enumxt>(), pruning, dual);
-  }
-  else
-  {
-    Enumerator enumerator(d, gso.get_mu_matrix(), gso.get_r_matrix());
-    FP_NR<mpfr_t> bestdist = -1;
-    while (enumerator.enum_next(max_dist))
-    {
-      if (flags & SVP_VERBOSE)
-      {
-        cerr << enumerator.get_sub_tree();
-        if (evaluator.eval_mode != EVALMODE_SV)
-          cerr << " (count=2*" << evaluator.size() << ")";
-      }
-
-      /* Enumerates short vectors only in enumerator.get_sub_tree()
-        (about maxVolume iterations or less) */
-      enumobj.enumerate(0, d, max_dist, 0, vector<FP_NR<mpfr_t>>(), enumerator.get_sub_tree(),
-                        pruning);
-
-      if (flags & SVP_VERBOSE)
-      {
-        cerr << "\r" << (char)27 << "[K";
-        if (evaluator.eval_mode == EVALMODE_SV && !evaluator.empty() &&
-            evaluator.begin()->first != bestdist)
-        {
-          bestdist = evaluator.begin()->first;
-          cerr << "Solution norm^2=" << bestdist << " value=" << evaluator.begin()->second << endl;
-        }
-      }
-    }
-  }
+  enumobj.enumerate(0, d, max_dist, 0, vector<FP_NR<mpfr_t>>(), vector<enumxt>(), pruning, dual);
   return !evaluator.empty();
 }
 
@@ -172,8 +154,8 @@ static int shortest_vector_ex(ZZ_mat<mpz_t> &b, vector<Z_NR<mpz_t>> &sol_coord,
   else if (method == SVPM_PROVED)
   {
     ExactErrorBoundedEvaluator *p = new ExactErrorBoundedEvaluator(
-        d, b, gso.get_mu_matrix(), gso.get_r_matrix(), eval_mode, max_aux_sols + 1,
-        EVALSTRATEGY_BEST_N_SOLUTIONS, findsubsols);
+        // d, b, gso.get_mu_matrix(), gso.get_r_matrix()
+        d, gso, eval_mode, max_aux_sols + 1, EVALSTRATEGY_BEST_N_SOLUTIONS, findsubsols);
     p->int_max_dist = int_max_dist;
     evaluator       = p;
   }
@@ -288,6 +270,221 @@ int shortest_vector_pruning(ZZ_mat<mpz_t> &b, vector<Z_NR<mpz_t>> &sol_coord,
   return shortest_vector_ex(b, sol_coord, SVPM_FAST, pruning, flags, EVALMODE_SV, tmp, nullptr,
                             nullptr, &auxsol_coord, &auxsol_dist, max_aux_sols);
 }
+
+//////////////////////////////////////////
+////// SVP FOR GSO OBJECT  ///////////////
+//////////////////////////////////////////
+
+static int shortest_vector_ex(
+    MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso, vector<Z_NR<mpz_t>> &sol_coord,
+    SVPMethod method, const vector<double> &pruning, int flags, EvaluatorMode eval_mode,
+    long long &sol_count, vector<vector<Z_NR<mpz_t>>> *subsol_coord = nullptr,
+    vector<enumf> *subsol_dist = nullptr, vector<vector<Z_NR<mpz_t>>> *auxsol_coord = nullptr,
+    vector<enumf> *auxsol_dist = nullptr, int max_aux_sols = 0, bool merge_sol_in_aux = false)
+{
+  bool findsubsols = (subsol_coord != nullptr) && (subsol_dist != nullptr);
+  bool findauxsols = (auxsol_coord != nullptr) && (auxsol_dist != nullptr) && (max_aux_sols != 0);
+
+  // d = lattice dimension (note that it might decrease during preprocessing)
+  // int d = b.get_rows();
+  int d = gso.d;  // Number of rows of b in the GSO
+
+  // n = dimension of the space
+  // int n = b.get_cols();
+  int n = gso.get_cols_of_b();  // number of columns of b in the GSO
+
+  FPLLL_CHECK(d > 0 && n > 0, "shortestVector: empty matrix");
+  FPLLL_CHECK(d <= n, "shortestVector: number of vectors > size of the vectors");
+
+  // Sets the floating-point precision
+  // Error bounds on GSO are valid if prec >= minprec
+  double rho;
+  int min_prec = gso_min_prec(rho, d, LLL_DEF_DELTA, LLL_DEF_ETA);
+  int prec     = max(53, min_prec + 10);
+  int old_prec = FP_NR<mpfr_t>::set_prec(prec);
+
+  // Allocates space for vectors and matrices in constructors
+  FP_NR<mpfr_t> max_dist;
+  Z_NR<mpz_t> int_max_dist;
+  Z_NR<mpz_t> itmp1;
+
+  // Computes the Gram-Schmidt orthogonalization in floating-point
+  gso.update_gso();
+  gen_zero_vect(sol_coord, d);
+
+  // If the last b_i* are too large, removes them to avoid an underflow
+  int new_d = last_useful_index(gso.get_r_matrix());
+  if (new_d < d)
+  {
+    // FPLLL_TRACE("Ignoring the last " << d - new_d << " vector(s)");
+    d = new_d;
+  }
+
+  if (flags & SVP_DUAL)
+  {
+    max_dist = 1.0 / gso.get_r_exp(d - 1, d - 1);
+    if (flags & SVP_VERBOSE)
+    {
+      cout << "max_dist = " << max_dist << endl;
+    }
+  }
+  else
+  {
+    // Computes a bound for the enumeration. This bound would work for an
+    //   exact algorithm, but we will increase it later to ensure that the fp
+    //   algorithm finds a solution
+
+    // Use the GSO version of get_basis_min
+    get_basis_min(int_max_dist, gso, 0, d);
+    max_dist.set_z(int_max_dist, GMP_RNDU);
+  }
+
+  // Initializes the evaluator of solutions
+  ErrorBoundedEvaluator *evaluator;
+  if (method == SVPM_FAST)
+  {
+    evaluator =
+        new FastErrorBoundedEvaluator(d, gso.get_mu_matrix(), gso.get_r_matrix(), eval_mode,
+                                      max_aux_sols + 1, EVALSTRATEGY_BEST_N_SOLUTIONS, findsubsols);
+  }
+  else if (method == SVPM_PROVED)
+  {
+    ExactErrorBoundedEvaluator *p = new ExactErrorBoundedEvaluator(
+        d, gso, eval_mode, max_aux_sols + 1, EVALSTRATEGY_BEST_N_SOLUTIONS, findsubsols);
+    p->int_max_dist = int_max_dist;
+    evaluator       = p;
+  }
+  else
+  {
+    FPLLL_ABORT("shortestVector: invalid evaluator type");
+  }
+  evaluator->init_delta_def(prec, rho, true);
+
+  if (!(flags & SVP_OVERRIDE_BND) && (eval_mode == EVALMODE_SV || method == SVPM_PROVED))
+  {
+    FP_NR<mpfr_t> ftmp1;
+    bool result = evaluator->get_max_error_aux(max_dist, true, ftmp1);
+    FPLLL_CHECK(result, "shortestVector: cannot compute an initial bound");
+    max_dist.add(max_dist, ftmp1, GMP_RNDU);
+  }
+
+  // Main loop of the enumeration
+  enumerate_svp(d, gso, max_dist, *evaluator, pruning, flags);  // Only uses r and mu
+
+  int result = RED_ENUM_FAILURE;
+  if (eval_mode != EVALMODE_SV)
+  {
+    result    = RED_SUCCESS;
+    sol_count = evaluator->sol_count * 2;
+  }
+  else if (!evaluator->empty())
+  {
+    // FP_NR<mpfr_t> fMaxError;
+    // validMaxError = evaluator->get_max_error(fMaxError);
+    // max_error = fMaxError.get_d(GMP_RNDU);
+    for (int i = 0; i < d; i++)
+    {
+      itmp1.set_f(evaluator->begin()->second[i]);
+      sol_coord[i].add(sol_coord[i], itmp1);
+    }
+    result = RED_SUCCESS;
+  }
+
+  if (findsubsols)
+  {
+    subsol_coord->clear();
+    subsol_dist->clear();
+    subsol_dist->resize(evaluator->sub_solutions.size());
+    for (size_t i = 0; i < evaluator->sub_solutions.size(); ++i)
+    {
+      (*subsol_dist)[i] = evaluator->sub_solutions[i].first.get_d();
+
+      vector<Z_NR<mpz_t>> ss_c;
+      for (size_t j = 0; j < evaluator->sub_solutions[i].second.size(); ++j)
+      {
+        itmp1.set_f(evaluator->sub_solutions[i].second[j]);
+        ss_c.emplace_back(itmp1);
+      }
+      subsol_coord->emplace_back(std::move(ss_c));
+    }
+  }
+  if (findauxsols)
+  {
+    auxsol_coord->clear();
+    auxsol_dist->clear();
+    // iterators over all solutions
+    auto it = evaluator->begin(), itend = evaluator->end();
+    // skip shortest solution
+    if (!merge_sol_in_aux)
+      ++it;
+    for (; it != itend; ++it)
+    {
+      auxsol_dist->push_back(it->first.get_d());
+
+      vector<Z_NR<mpz_t>> as_c;
+      for (size_t j = 0; j < it->second.size(); ++j)
+      {
+        itmp1.set_f(it->second[j]);
+        as_c.emplace_back(itmp1);
+      }
+      auxsol_coord->emplace_back(std::move(as_c));
+    }
+  }
+
+  delete evaluator;
+  FP_NR<mpfr_t>::set_prec(old_prec);
+  return result;
+}
+
+int shortest_vector(MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso,
+                    vector<Z_NR<mpz_t>> &sol_coord, SVPMethod method, int flags)
+{
+  long long tmp;
+  return shortest_vector_ex(gso, sol_coord, method, vector<double>(), flags, EVALMODE_SV, tmp);
+}
+
+int shortest_vectors(MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso,
+                     vector<vector<Z_NR<mpz_t>>> &sol_coord, vector<enumf> &sol_dist,
+                     const int max_sols, SVPMethod method, int flags)
+{
+  long long tmp;
+  vector<Z_NR<mpz_t>> sol_coord_tmp;
+  return shortest_vector_ex(gso, sol_coord_tmp, method, vector<double>(), flags, EVALMODE_SV, tmp,
+                            nullptr, nullptr, &sol_coord, &sol_dist, max_sols - 1, true);
+}
+
+int shortest_vector_pruning(MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso,
+                            vector<Z_NR<mpz_t>> &sol_coord, const vector<double> &pruning,
+                            int flags)
+{
+  long long tmp;
+  return shortest_vector_ex(gso, sol_coord, SVPM_FAST, pruning, flags, EVALMODE_SV, tmp);
+}
+
+int shortest_vector_pruning(MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso,
+                            vector<Z_NR<mpz_t>> &sol_coord,
+                            vector<vector<Z_NR<mpz_t>>> &subsol_coord, vector<enumf> &subsol_dist,
+                            const vector<double> &pruning, int flags)
+{
+  long long tmp;
+  return shortest_vector_ex(gso, sol_coord, SVPM_FAST, pruning, flags, EVALMODE_SV, tmp,
+                            &subsol_coord, &subsol_dist);
+}
+
+int shortest_vector_pruning(MatGSOInterface<Z_NR<mpz_t>, FP_NR<mpfr_t>> &gso,
+                            vector<Z_NR<mpz_t>> &sol_coord,
+                            vector<vector<Z_NR<mpz_t>>> &auxsol_coord, vector<enumf> &auxsol_dist,
+                            const int max_aux_sols, const vector<double> &pruning, int flags)
+{
+  long long tmp;
+  return shortest_vector_ex(gso, sol_coord, SVPM_FAST, pruning, flags, EVALMODE_SV, tmp, nullptr,
+                            nullptr, &auxsol_coord, &auxsol_dist, max_aux_sols);
+}
+
+///////////////////////////////////
+//////END SVP FOR GSO OBJECT //////
+///////////////////////////////////
+
 /* Closest vector problem
    ====================== */
 
@@ -428,8 +625,8 @@ int closest_vector(ZZ_mat<mpz_t> &b, const vector<Z_NR<mpz_t>> &int_target,
           max_index = cur;
         }
       }
-      for (cur                        = max_index; cur < previous_max_index; ++cur)
-        max_indices[cur]              = max_index;
+      for (cur = max_index; cur < previous_max_index; ++cur)
+        max_indices[cur] = max_index;
       max_indices[previous_max_index] = previous_max_index;
       previous_max_index              = max_index;
       --max_index;
