@@ -13,7 +13,7 @@
    You should have received a copy of the GNU Lesser General Public License
    along with fplll. If not, see <http://www.gnu.org/licenses/>. */
 
-#include <fplll/threadpool.h>
+#include "threadpool.h"
 
 FPLLL_BEGIN_NAMESPACE
 
