@@ -13,9 +13,9 @@
    You should have received a copy of the GNU Lesser General Public License
    along with fplll. If not, see <http://www.gnu.org/licenses/>. */
 
+#include "test_utils.h"
 #include <cstring>
-#include <fplll.h>
-#include <test_utils.h>
+#include <fplll/fplll.h>
 
 #ifndef TESTDATADIR
 #define TESTDATADIR ".."
