@@ -14,9 +14,9 @@
    You should have received a copy of the GNU Lesser General Public License
    along with fplll. If not, see <http://www.gnu.org/licenses/>. */
 
+#include "test_utils.h"
 #include <cstring>
-#include <fplll.h>
-#include <test_utils.h>
+#include <fplll/fplll.h>
 
 #ifndef TESTDATADIR
 #define TESTDATADIR ".."
@@ -274,7 +274,9 @@ template <class ZT> int test_dsvp_reduce(ZZ_mat<ZT> &A, vector<Z_NR<mpz_t>> &b)
 int test_svp_proved_vs_fast(size_t d, size_t d2)
 {
   ZZ_mat<mpz_t> A(d, d);
-  A.gen_qary_withq(d / 2, 65537);
+  Z_NR<mpz_t> q;
+  q = 65537;
+  A.gen_qary(d / 2, q);
 
   int r = lll_reduction(A);
   ZZ_mat<mpz_t> B(d2, d);  // so that LLL doesn't just solve it
